


function[out] = implicit_solver(neuron_flag, statevars, g_m, Ei, z, dt, Ielec,t)



% this solver for the multi-compartment model follows the conventions in
% Dayan & Abbott - Theoretical Neuroscience (Appendix to Chapter 6)



%% --- neuron sizes --- %%

if  (neuron_flag == 1)         %
    areaS   = 4*pi*0.01^2;     % EIN_L - excitatory interneuron
end                            %
if  (neuron_flag == 2)         %
    areaS   = 4*pi*0.04^2;     % LIN_L - lateral interneurons
end                            %
if  (neuron_flag == 3)         %
    areaS   = 4*pi*0.015^2;    % CCIN_L - cross-inhibitory interneurons
end                            %
if  (neuron_flag == 5)         %
    areaS   = 4*pi*0.01^2;     % EIN_R - excitatory interneuron
end                            %
if  (neuron_flag == 6)         %
    areaS   = 4*pi*0.04^2;     % LIN_R - lateral interneurons
end                            %
if  (neuron_flag == 7)         %
    areaS   = 4*pi*0.015^2;    % CCIN_R - cross-inhibitory interneurons
end                            
                               
area_IS    =     0.1*areaS;    
area_DEN   =       4*areaS;    
                              
        

%% --- neuron parameters --- %%                         
                               
CAPmCOMP0  =    10*area_IS;     % capacitance
CAPmSOMA   =      10*areaS;     
CAPmDEND   =   10*area_DEN;     
CONcore    =     4.2*areaS;    
CONcore_IS =   300*area_IS;    
CONcore_D  =  4.2*area_DEN;    
                               
Cm_1 = CAPmCOMP0;              
Cm_2 = CAPmSOMA;               
Cm_3 = CAPmDEND;               
Cm_4 = CAPmDEND;               
Cm_5 = CAPmDEND;               
                               
g_0and1 = 0;                     % conductances between compartments    
g_1and2 = CONcore_IS;          
g_2and3 = CONcore;             
g_3and4 = CONcore;             
g_4and5 = CONcore;             
g_5and6 = 0;                   
                               
V1 = statevars(12,2);          % IS at t old - membrane potential
V2 = statevars(1,2);           % soma at t old
V3 = statevars(2,2);           % ...
V4 = statevars(3,2);           % ...
V5 = statevars(4,2);           % ...
                               
g1_tot = g_m(1,1) + g_m(1,2) + g_m(1,3) + g_m(1,4);  
g2_tot = g_m(2,1) + g_m(2,2) + g_m(2,3) + g_m(2,4) + g_m(2,5) + g_m(2,6) + g_m(2,7) + g_m(2,8);
g3_tot = g_m(3,1) + g_m(3,2) + g_m(3,3) + g_m(3,4) + g_m(3,5);
g4_tot = g_m(4,1) + g_m(4,2) + g_m(4,3);
g5_tot = g_m(5,1) + g_m(5,2);         
                               
g_m = round(g_m*10000000)/10000000;                      
Ei = round(Ei*10000000)/10000000;   

g_mE = g_m.*Ei;  

g_mE = round(g_mE*10000000)/10000000;
                               
gE1_tot = g_mE(1,1) + g_mE(1,2) + g_mE(1,3) + g_mE(1,4); 
gE2_tot = g_mE(2,1) + g_mE(2,2) + g_mE(2,3) + g_mE(2,4) + g_mE(2,5) + g_mE(2,6) + g_mE(2,7) + g_mE(2,8);
gE3_tot = g_mE(3,1) + g_mE(3,2) + g_mE(3,3) + g_mE(3,4) + g_mE(3,5);
gE4_tot = g_mE(4,1) + g_mE(4,2) + g_mE(4,3);
gE5_tot = g_mE(5,1) + g_mE(5,2);
    


%% --- iteration parameters --- %%
                     
% for the meaning of these coefficients, see % Dayan & Abbott - Theoretical Neuroscience (Appendix to Chapter 6)

%B1 = 1/Cm_1 * g_0and1;          
B2 = 1/Cm_2 * g_1and2;           
B3 = 1/Cm_3 * g_2and3;           
B4 = 1/Cm_4 * g_3and4;          
B5 = 1/Cm_5 * g_4and5;           
                                
C1 = -1/Cm_1 * (g_0and1 + g_1and2 + g1_tot);
C2 = -1/Cm_2 * (g_1and2 + g_2and3 + g2_tot);
C3 = -1/Cm_3 * (g_2and3 + g_3and4 + g3_tot);
C4 = -1/Cm_4 * (g_3and4 + g_4and5 + g4_tot);
C5 = -1/Cm_5 * (g_4and5 + g_5and6 + g5_tot);

D1 = 1/Cm_1 * g_1and2;
D2 = 1/Cm_2 * g_2and3;
D3 = 1/Cm_3 * g_3and4;
D4 = 1/Cm_4 * g_4and5;
%D5 = 1/Cm_5 * g_5and6;

F1 = 1/Cm_1 *  (gE1_tot + 0);
F2 = 1/Cm_2 *  (gE2_tot + Ielec); 
F3 = 1/Cm_3 *  gE3_tot;
F4 = 1/Cm_4 *  gE4_tot;
F5 = 1/Cm_5 *  gE5_tot;

%b1 = B1*z*dt;
b2 = B2*z*dt;
b3 = B3*z*dt;
b4 = B4*z*dt;
b5 = B5*z*dt;

c1 = C1*z*dt;                    
c2 = C2*z*dt;
c3 = C3*z*dt;
c4 = C4*z*dt;
c5 = C5*z*dt;

d1 = D1*z*dt;
d2 = D2*z*dt;
d3 = D3*z*dt;
d4 = D4*z*dt;
%d5 = D5*z*dt;                   

f1 = (F1 +         C1*V1 + D1*V2)*dt;   
f2 = (F2 + B2*V1 + C2*V2 + D2*V3)*dt;
f3 = (F3 + B3*V2 + C3*V3 + D3*V4)*dt;
f4 = (F4 + B4*V3 + C4*V4 + D4*V5)*dt;
f5 = (F5 + B5*V4 + C5*V5        )*dt;   
 
c_1 = c1 ;
c_2 = c2 + b2*d1/(1 - c_1);
c_3 = c3 + b3*d2/(1 - c_2);
c_4 = c4 + b4*d3/(1 - c_3);
c_5 = c5 + b5*d4/(1 - c_4);
   
f_1 = f1;
f_1 = round(f_1*1000000)/1000000;  

f_2 = f2 + b2*f_1/(1 - c_1);     
f_2 = round(f_2*1000000)/1000000;

f_3 = f3 + b3*f_2/(1 - c_2);
f_3 = round(f_3*1000000)/1000000;

f_4 = f4 + b4*f_3/(1 - c_3);
f_4 = round(f_4*1000000)/1000000;

f_5 = f5 + b5*f_4/(1 - c_4);
f_5 = round(f_5*1000000)/1000000;

dV5 = f_5/(1 - c_5);
dV4 = (d4*dV5 + f_4) / (1 - c_4);
dV3 = (d3*dV4 + f_3) / (1 - c_3);
dV2 = (d2*dV3 + f_2) / (1 - c_2);
dV1 = (d1*dV2 + f_1) / (1 - c_1); 
 
statevars(12,1) = statevars(12,2) + dV1;
statevars(1,1)  =  statevars(1,2) + dV2;
statevars(2,1)  =  statevars(2,2) + dV3;
statevars(3,1)  =  statevars(3,2) + dV4;
statevars(4,1)  =  statevars(4,2) + dV5;



%% --- update --- %%  

out = statevars;     
                     
end